/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.testing.script;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import net.sourceforge.htmlunit.corejs.javascript.Context;
import net.sourceforge.htmlunit.corejs.javascript.JavaScriptException;
import net.sourceforge.htmlunit.corejs.javascript.RhinoException;
import net.sourceforge.htmlunit.corejs.javascript.Script;
import net.sourceforge.htmlunit.corejs.javascript.Scriptable;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Before;
import org.junit.Test;

@SuppressWarnings("nls")
public abstract class AbstractRhinoScriptTest {

  protected List<String> getScriptResourceNames() throws IOException {
    final String scriptsResourceName = getScriptsResourceName();

    return getScriptResourceNames(scriptsResourceName);

  }

  protected List<String> getScriptResourceNames(final String scriptsResourceName)
      throws IOException {
    InputStream is = null;
    try {
      is = getClass().getResourceAsStream(scriptsResourceName);
      if (is == null) {
        throw new IOException("Could not load scripts resource [" + scriptsResourceName + "].");
      }
      @SuppressWarnings("unchecked")
      final List<String> scriptResourceNames = IOUtils.readLines(is);
      final List<String> results = new ArrayList<String>();
      for (final String name : scriptResourceNames) {
        if (name.endsWith(".scripts")) {
          results.addAll(getScriptResourceNames(name));
        }
        else {
          results.add(name);
        }
      }
      return results;
    }
    finally {
      IOUtils.closeQuietly(is);
    }
  }

  protected String getScriptsResourceName() {
    final String scriptsResourceName = "/" + getClass().getName().replace('.', '/') + ".scripts";
    return scriptsResourceName;
  }

  private Context context;

  private List<Script> compiledScripts;

  @Before
  public void before() throws Exception {
    this.context = Context.enter();

    this.compiledScripts = compileScripts(context, getScriptResourceNames());
  }

  protected Map<String, Object> getImplicitObjects() {
    final Map<String, Object> implicitObjects = new HashMap<String, Object>();

    final Log logger = LogFactory.getLog(getClass());
    implicitObjects.put("logger", logger);
    return implicitObjects;
  }

  @Test
  public void run() throws RhinoException {
    try {
      final Scriptable scope = getScope();
      final Map<String, Object> implicitObjects = getImplicitObjects();
      for (Entry<String, Object> entry : implicitObjects.entrySet()) {
        scope.put(entry.getKey(), scope, entry.getValue());
      }

      for (Script compiledScript : this.compiledScripts) {
        compiledScript.exec(context, scope);
      }
    }
    catch (JavaScriptException jsex) {
      throw jsex;
    }
  }

  protected Scriptable getScope() {
    final Scriptable scope = context.initStandardObjects();
    return scope;
  }

  public void after() {
    Context.exit();
  }

  protected List<Script> compileScripts(
      final Context context,
      final List<String> scriptResourceNames) throws RhinoException, IOException {
    final List<Script> compiledScripts = new LinkedList<Script>();
    for (String scriptResourceName : scriptResourceNames) {
      InputStream is = null;
      try {
        is = getClass().getResourceAsStream(scriptResourceName);
        if (is == null) {
          throw new IOException("Could not load script resource [" + scriptResourceName + "].");
        }
        Script script = context.compileReader(
            new InputStreamReader(is, "UTF-8"),
            scriptResourceName,
            1,
            null);
        compiledScripts.add(script);
      }
      finally {
        IOUtils.closeQuietly(is);
      }
    }
    return compiledScripts;
  }
}
