/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.testing.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class PropertyResolver {

  private final Map<String, String> properties;

  public PropertyResolver(Map<String, String> properties) {
    this.properties = (properties == null ? Collections.<String, String> emptyMap() : properties);
  }

  public Map<String, String> resolve(Map<String, String> properties) {
    if (properties == null) {
      return null;
    }
    else {

      final Map<String, String> resolvingProperties = new HashMap<String, String>(this.properties);
      resolvingProperties.putAll(properties);

      Map<String, String> currentProperties = properties;
      boolean changed = true;
      while (changed) {
        final Map<String, String> resolvedProperties = new HashMap<String, String>(currentProperties.size());
        changed = false;
        for (Entry<String, String> entry : currentProperties.entrySet()) {
          final String value = entry.getValue();
          final String resolvedValue = resolve(value, resolvingProperties);
          if (value != resolvedValue && (value == null || value.equals(resolvedValue))) {
            changed = true;
          }
          resolvedProperties.put(entry.getKey(), resolvedValue);
        }
        currentProperties = resolvedProperties;
      }
      return currentProperties;
    }
  }

  protected String resolve(String value, Map<String, String> properties) {

    if (value == null) {
      return null;
    }
    else {
      final int start = value.indexOf("${");
      final int end = value.indexOf("}");
      if (start >= 0 && end >= 0 && start < end) {
        final String name = value.substring(start + 2, end);
        final String targetValue = properties.get(name);
        if (targetValue == null) {
          return value;
        }
        else {

          final String prefix = value.substring(0, start);
          final String suffix = (end == value.length() ? "" : value.substring(end + 1));

          return resolve(prefix + targetValue + suffix, properties);
        }
      }
      else {
        return value;
      }
    }

  }

}
